@@ -6,7 +6,7 @@ import (
 	"reflect"
 	"strings"
 
-	"github.com/sirupsen/logrus"
+	"github.com/openshift/elasticsearch-operator/pkg/log"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/apimachinery/pkg/types"
@@ -25,42 +25,52 @@ var DISK_WATERMARK_HIGH_PCT *float64
 var DISK_WATERMARK_LOW_ABS *resource.Quantity
 var DISK_WATERMARK_HIGH_ABS *resource.Quantity
 
-func (elasticsearchRequest *ElasticsearchRequest) UpdateClusterStatus() error {
+func (er *ElasticsearchRequest) UpdateClusterStatus() error {
 
-	cluster := elasticsearchRequest.cluster
-	esClient := elasticsearchRequest.esClient
+	cluster := er.cluster
+	esClient := er.esClient
 
 	clusterStatus := cluster.Status.DeepCopy()
 
-	health, err := esClient.GetClusterHealth()
-	if err != nil {
-		health.Status = healthUnknown
+	health := api.ClusterHealth{
+		Status: healthUnknown,
 	}
+
+	// if the cluster isn't ready don't both to try to curl it
+	if er.AnyNodeReady() {
+		health, _ = esClient.GetClusterHealth()
+	}
+
 	clusterStatus.Cluster = health
+	clusterStatus.ShardAllocationEnabled = api.ShardAllocationUnknown
 
-	allocation, err := esClient.GetShardAllocation()
-	switch {
-	case allocation == "none":
-		clusterStatus.ShardAllocationEnabled = api.ShardAllocationNone
-	case allocation == "primaries":
-		clusterStatus.ShardAllocationEnabled = api.ShardAllocationPrimaries
-	case allocation == "all":
-		clusterStatus.ShardAllocationEnabled = api.ShardAllocationAll
-	default:
-		clusterStatus.ShardAllocationEnabled = api.ShardAllocationUnknown
+	// if the cluster isn't ready don't both to try to curl it
+	if er.AnyNodeReady() {
+		allocation, _ := esClient.GetShardAllocation()
+		switch {
+		case allocation == "none":
+			clusterStatus.ShardAllocationEnabled = api.ShardAllocationNone
+		case allocation == "primaries":
+			clusterStatus.ShardAllocationEnabled = api.ShardAllocationPrimaries
+		case allocation == "all":
+			clusterStatus.ShardAllocationEnabled = api.ShardAllocationAll
+		default:
+			clusterStatus.ShardAllocationEnabled = api.ShardAllocationUnknown
+		}
 	}
 
-	clusterStatus.Pods = rolePodStateMap(cluster.Namespace, cluster.Name, elasticsearchRequest.client)
+	clusterStatus.Pods = rolePodStateMap(cluster.Namespace, cluster.Name, er.client)
 	updateStatusConditions(clusterStatus)
-	elasticsearchRequest.updateNodeConditions(clusterStatus)
+	if err := er.updateNodeConditions(clusterStatus); err != nil {
+		return err
+	}
 
 	if !reflect.DeepEqual(clusterStatus, cluster.Status) {
 		nretries := -1
 		retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
 			nretries++
-			if getErr := elasticsearchRequest.client.Get(context.TODO(), types.NamespacedName{Name: cluster.Name, Namespace: cluster.Namespace}, cluster); getErr != nil {
-				logrus.Debugf("Could not get Elasticsearch %v: %v", cluster.Name, getErr)
-				return getErr
+			if err := er.client.Get(context.TODO(), types.NamespacedName{Name: cluster.Name, Namespace: cluster.Namespace}, cluster); err != nil {
+				return err
 			}
 
 			cluster.Status.Cluster = clusterStatus.Cluster
@@ -69,9 +79,8 @@ func (elasticsearchRequest *ElasticsearchRequest) UpdateClusterStatus() error {
 			cluster.Status.ShardAllocationEnabled = clusterStatus.ShardAllocationEnabled
 			cluster.Status.Nodes = clusterStatus.Nodes
 
-			if updateErr := elasticsearchRequest.client.Update(context.TODO(), cluster); updateErr != nil {
-				logrus.Debugf("Failed to update Elasticsearch %s status. Reason: %v. Trying again...", cluster.Name, updateErr)
-				return updateErr
+			if err := er.client.Status().Update(context.TODO(), cluster); err != nil {
+				return err
 			}
 			return nil
 		})
@@ -79,14 +88,57 @@ func (elasticsearchRequest *ElasticsearchRequest) UpdateClusterStatus() error {
 		if retryErr != nil {
 			return fmt.Errorf("Error: could not update status for Elasticsearch %v after %v retries: %v", cluster.Name, nretries, retryErr)
 		}
-		logrus.Debugf("Updated Elasticsearch %v after %v retries", cluster.Name, nretries)
 	}
 
 	return nil
 }
 
-func (elasticsearchRequest *ElasticsearchRequest) GetCurrentPodStateMap() map[api.ElasticsearchNodeRole]api.PodStateMap {
-	return rolePodStateMap(elasticsearchRequest.cluster.Namespace, elasticsearchRequest.cluster.Name, elasticsearchRequest.client)
+func (er *ElasticsearchRequest) GetCurrentPodStateMap() map[api.ElasticsearchNodeRole]api.PodStateMap {
+	return rolePodStateMap(er.cluster.Namespace, er.cluster.Name, er.client)
+}
+
+func (er *ElasticsearchRequest) setNodeStatus(node NodeTypeInterface, nodeStatus *api.ElasticsearchNodeStatus, clusterStatus *api.ElasticsearchStatus) error {
+
+	index, _ := getNodeStatus(node.name(), clusterStatus)
+
+	if index == NOT_FOUND_INDEX {
+		clusterStatus.Nodes = append(clusterStatus.Nodes, *nodeStatus)
+	} else {
+		clusterStatus.Nodes[index] = *nodeStatus
+	}
+
+	return er.updateNodeStatus(*clusterStatus)
+}
+
+func (er *ElasticsearchRequest) updateNodeStatus(status api.ElasticsearchStatus) error {
+
+	cluster := er.cluster
+	// if there is nothing to update, don't
+	if reflect.DeepEqual(cluster.Status, status) {
+		return nil
+	}
+
+	nretries := -1
+	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
+		nretries++
+		if err := er.client.Get(context.TODO(), types.NamespacedName{Name: cluster.Name, Namespace: cluster.Namespace}, cluster); err != nil {
+			return err
+		}
+
+		cluster.Status = status
+
+		if err := er.client.Status().Update(context.TODO(), cluster); err != nil {
+			return err
+		}
+
+		return nil
+	})
+
+	if retryErr != nil {
+		return fmt.Errorf("Error: could not update status for Elasticsearch %v after %v retries: %v", cluster.Name, nretries, retryErr)
+	}
+
+	return nil
 }
 
 func containsClusterCondition(condition api.ClusterConditionType, status v1.ConditionStatus, elasticsearchStatus *api.ElasticsearchStatus) bool {
@@ -192,13 +244,17 @@ func isPodReady(pod v1.Pod) bool {
 
 func (er *ElasticsearchRequest) updateNodeConditions(status *api.ElasticsearchStatus) error {
 	esClient := er.esClient
-	cluster := er.cluster
 
 	// Get all pods based on status.Nodes[] and check their conditions
 	// get pod with label 'node-name=node.getName()'
-	thresholdEnabled, err := esClient.GetThresholdEnabled()
-	if err != nil {
-		logrus.Debugf("Unable to check if threshold is enabled for %v", cluster.Name)
+	thresholdEnabled := false
+	if er.AnyNodeReady() {
+		var err error
+
+		thresholdEnabled, err = esClient.GetThresholdEnabled()
+		if err != nil {
+			er.L().Info("Unable to check if threshold is enabled", "error", err)
+		}
 	}
 
 	if thresholdEnabled {
@@ -219,11 +275,13 @@ func (er *ElasticsearchRequest) updateNodeConditions(status *api.ElasticsearchSt
 
 func (er *ElasticsearchRequest) pruneMissingNodes(status *api.ElasticsearchStatus) error {
 	cluster := er.cluster
+	ll := er.L()
 
 	clusterNodes := nodes[nodeMapKey(er.cluster.GetName(), er.cluster.GetNamespace())]
 
 	ns := status.Nodes[:0]
 	for _, nodeStatus := range status.Nodes {
+		ll = ll.WithValues("node", nodeStatus)
 
 		nodeName := "unknown name"
 		if nodeStatus.DeploymentName != "" {
@@ -284,8 +342,10 @@ func (er *ElasticsearchRequest) pruneMissingNodes(status *api.ElasticsearchStatu
 
 func (er *ElasticsearchRequest) updatePodNodeConditions(status *api.ElasticsearchStatus, thresholdEnabled bool) error {
 	cluster := er.cluster
+	ll := er.L()
 
 	for nodeIndex, node := range status.Nodes {
+		ll = ll.WithValues("node", node)
 
 		nodeName := "unknown name"
 		if node.DeploymentName != "" {
@@ -310,6 +370,7 @@ func (er *ElasticsearchRequest) updatePodNodeConditions(status *api.Elasticsearc
 		}
 
 		for _, nodePod := range nodePodList.Items {
+
 			isUnschedulable := false
 			for _, podCondition := range nodePod.Status.Conditions {
 				if podCondition.Type == v1.PodScheduled && podCondition.Status == v1.ConditionFalse {
@@ -402,7 +463,7 @@ func (er *ElasticsearchRequest) updatePodNodeConditions(status *api.Elasticsearc
 
 			usage, percent, err := er.esClient.GetNodeDiskUsage(nodeName)
 			if err != nil {
-				logrus.Infof("Unable to get disk usage: %s", err)
+				ll.Info("Unable to get disk usage", "error", err)
 				continue
 			}
 
@@ -436,7 +497,7 @@ func (er *ElasticsearchRequest) refreshDiskWatermarkThresholds() {
 	//quantity, err := resource.ParseQuantity(string)
 	low, high, err := er.esClient.GetDiskWatermarks()
 	if err != nil {
-		logrus.Debugf("Unable to refresh disk watermarks from cluster, using defaults")
+		er.L().Info("Unable to refresh disk watermarks from cluster, using defaults", "error", err)
 	}
 
 	switch low.(type) {
@@ -447,13 +508,12 @@ func (er *ElasticsearchRequest) refreshDiskWatermarkThresholds() {
 	case string:
 		value, err := resource.ParseQuantity(strings.ToUpper(low.(string)))
 		if err != nil {
-			logrus.Warnf("Unable to parse %v: %v", low.(string), err)
+			er.L().Info("Unable to parse quantity", "value", low.(string), "error", err)
 		}
 		DISK_WATERMARK_LOW_ABS = &value
 		DISK_WATERMARK_LOW_PCT = nil
 	default:
-		// error
-		logrus.Warnf("Unknown type for low: %T", low)
+		er.L().Error(err, "Unknown type for low", "type", fmt.Sprintf("%T", low))
 	}
 
 	switch high.(type) {
@@ -464,13 +524,13 @@ func (er *ElasticsearchRequest) refreshDiskWatermarkThresholds() {
 	case string:
 		value, err := resource.ParseQuantity(strings.ToUpper(high.(string)))
 		if err != nil {
-			logrus.Warnf("Unable to parse %v: %v", high.(string), err)
+			er.L().Info("Unable to parse quantity", "value", high.(string), "error", err)
 		}
 		DISK_WATERMARK_HIGH_ABS = &value
 		DISK_WATERMARK_HIGH_PCT = nil
 	default:
 		// error
-		logrus.Warnf("Unknown type for high: %T", high)
+		er.L().Error(err, "Unknown type for high", "type", fmt.Sprintf("%T", high))
 	}
 
 }
@@ -493,7 +553,7 @@ func exceedsWatermarks(usage string, percent float64, watermarkUsage *resource.Q
 
 	quantity, err := resource.ParseQuantity(usage)
 	if err != nil {
-		logrus.Warnf("Unable to parse usage quantity %v: %v", usage, err)
+		log.Error(err, "Unable to parse quantity", "value", usage)
 		return false
 	}
 
@@ -600,7 +660,7 @@ func updatePodNodeStorageCondition(node *api.ElasticsearchNodeStatus, reason, me
 
 func updateStatusConditions(status *api.ElasticsearchStatus) {
 	if status.Conditions == nil {
-		status.Conditions = make([]api.ClusterCondition, 0, 4)
+		status.Conditions = make([]api.ClusterCondition, 0, 6)
 	}
 	if _, condition := getESNodeCondition(status.Conditions, api.UpdatingSettings); condition == nil {
 		updateUpdatingSettingsCondition(status, v1.ConditionFalse)
@@ -614,6 +674,12 @@ func updateStatusConditions(status *api.ElasticsearchStatus) {
 	if _, condition := getESNodeCondition(status.Conditions, api.Restarting); condition == nil {
 		updateRestartingCondition(status, v1.ConditionFalse)
 	}
+	if _, condition := getESNodeCondition(status.Conditions, api.Recovering); condition == nil {
+		updateRecoveringCondition(status, v1.ConditionFalse)
+	}
+	if _, condition := getESNodeCondition(status.Conditions, api.UpdatingESSettings); condition == nil {
+		updateUpdatingESSettingsCondition(status, v1.ConditionFalse)
+	}
 }
 
 func isPodUnschedulableConditionTrue(conditions []api.ClusterCondition) bool {
@@ -694,16 +760,18 @@ func updateESNodeCondition(status *api.ElasticsearchStatus, condition *api.Clust
 func updateConditionWithRetry(dpl *api.Elasticsearch, value v1.ConditionStatus,
 	executeUpdateCondition func(*api.ElasticsearchStatus, v1.ConditionStatus) bool, client client.Client) error {
 	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
-		if getErr := client.Get(context.TODO(), types.NamespacedName{Name: dpl.Name, Namespace: dpl.Namespace}, dpl); getErr != nil {
-			logrus.Debugf("Could not get Elasticsearch %v: %v", dpl.Name, getErr)
-			return getErr
+		if err := client.Get(context.TODO(), types.NamespacedName{Name: dpl.Name, Namespace: dpl.Namespace}, dpl); err != nil {
+			log.Info("Could not get Elasticsearch", "cluster", dpl.Name, "error", err)
+			return err
 		}
 
-		executeUpdateCondition(&dpl.Status, value)
+		if changed := executeUpdateCondition(&dpl.Status, value); !changed {
+			return nil
+		}
 
-		if updateErr := client.Update(context.TODO(), dpl); updateErr != nil {
-			logrus.Debugf("Failed to update Elasticsearch %v status: %v", dpl.Name, updateErr)
-			return updateErr
+		if err := client.Status().Update(context.TODO(), dpl); err != nil {
+			log.Info("Failed to update Elasticsearch status", "cluster", dpl.Name, "error", err)
+			return err
 		}
 		return nil
 	})
@@ -814,3 +882,17 @@ func updateRestartingCondition(status *api.ElasticsearchStatus, value v1.Conditi
 		Status: value,
 	})
 }
+
+func updateRecoveringCondition(status *api.ElasticsearchStatus, value v1.ConditionStatus) bool {
+	return updateESNodeCondition(status, &api.ClusterCondition{
+		Type:   api.Recovering,
+		Status: value,
+	})
+}
+
+func updateUpdatingESSettingsCondition(status *api.ElasticsearchStatus, value v1.ConditionStatus) bool {
+	return updateESNodeCondition(status, &api.ClusterCondition{
+		Type:   api.UpdatingESSettings,
+		Status: value,
+	})
+}
